@@ -11,6 +11,7 @@ from websockets.sync.client import connect, Connection
 from websockets.exceptions import ConnectionClosedOK
 
 from ..base_client import BaseElevenLabs
+from ..version import __version__
 
 
 class ClientToOrchestratorEvent(str, Enum):
@@ -222,10 +223,12 @@ class ConversationInitiationData:
         extra_body: Optional[dict] = None,
         conversation_config_override: Optional[dict] = None,
         dynamic_variables: Optional[dict] = None,
+        user_id: Optional[str] = None,
     ):
         self.extra_body = extra_body or {}
         self.conversation_config_override = conversation_config_override or {}
         self.dynamic_variables = dynamic_variables or {}
+        self.user_id = user_id
 
 
 class Conversation:
@@ -239,6 +242,7 @@ class Conversation:
     callback_agent_response_correction: Optional[Callable[[str, str], None]]
     callback_user_transcript: Optional[Callable[[str], None]]
     callback_latency_measurement: Optional[Callable[[int], None]]
+    callback_end_session: Optional[Callable]
 
     _thread: Optional[threading.Thread]
     _should_stop: threading.Event
@@ -260,6 +264,7 @@ class Conversation:
         callback_agent_response_correction: Optional[Callable[[str, str], None]] = None,
         callback_user_transcript: Optional[Callable[[str], None]] = None,
         callback_latency_measurement: Optional[Callable[[int], None]] = None,
+        callback_end_session: Optional[Callable] = None,
     ):
         """Conversational AI session.
 
@@ -291,6 +296,7 @@ class Conversation:
         self.callback_agent_response_correction = callback_agent_response_correction
         self.callback_user_transcript = callback_user_transcript
         self.callback_latency_measurement = callback_latency_measurement
+        self.callback_end_session = callback_end_session
 
         self.client_tools.start()
 
@@ -316,6 +322,9 @@ class Conversation:
         self._ws = None
         self._should_stop.set()
 
+        if self.callback_end_session:
+            self.callback_end_session()
+
     def wait_for_session_end(self) -> Optional[str]:
         """Waits for the conversation session to end.
 
@@ -397,6 +406,11 @@ class Conversation:
                         "custom_llm_extra_body": self.config.extra_body,
                         "conversation_config_override": self.config.conversation_config_override,
                         "dynamic_variables": self.config.dynamic_variables,
+                        "source_info": {
+                            "source": "python_sdk",
+                            "version": __version__,
+                        },
+                        **({"user_id": self.config.user_id} if self.config.user_id else {}),
                     }
                 )
             )
@@ -491,8 +505,11 @@ class Conversation:
 
     def _get_wss_url(self):
         base_ws_url = self.client._client_wrapper.get_environment().wss
-        return f"{base_ws_url}/v1/convai/conversation?agent_id={self.agent_id}"
+        return f"{base_ws_url}/v1/convai/conversation?agent_id={self.agent_id}&source=python_sdk&version={__version__}"
 
     def _get_signed_url(self):
         response = self.client.conversational_ai.conversations.get_signed_url(agent_id=self.agent_id)
-        return response.signed_url
+        signed_url = response.signed_url
+        # Append source and version query parameters to the signed URL
+        separator = "&" if "?" in signed_url else "?"
+        return f"{signed_url}{separator}source=python_sdk&version={__version__}"
